<?php

namespace Drupal\media_thumbnails_svg\Plugin\MediaThumbnail;

use Drupal\Core\Entity\EntityStorageException;
use Drupal\file\Entity\File;
use Drupal\media_thumbnails\Plugin\MediaThumbnailBase;
use SVG\SVG;

/**
 * Media thumbnail plugin for svg documents.
 *
 * @MediaThumbnail(
 *   id = "media_thumbnail_svg",
 *   label = @Translation("Media Thumbnail SVG"),
 *   mime = {
 *     "image/svg+xml",
 *   }
 * )
 */
class MediaThumbnailSVG extends MediaThumbnailBase {

  /**
   * Creates a managed thumbnail file using the passed source file uri.
   *
   * {@inheritdoc}
   */
  public function createThumbnail($sourceUri) {

    $image = SVG::fromFile($sourceUri);
    if (!$image) {
      $this->logger->warning($this->t('Media entity source file (svg) not found.'));
      return NULL;
    }

    // Create a raster image using the target width, keeping the aspect ratio.
    $width = $image->getDocument()->getWidth() ?: $image->getDocument()->getViewBox()[2];
    $height = $image->getDocument()->getHeight() ?: $image->getDocument()->getViewBox()[3];
    $ratio = $width && $height ? $height / $width : 1;
    $width = $this->configuration['width'] ?? 500;
    $height = (int) ($width * $ratio);
    $raster_image = $image->toRasterImage($width, $height);

    // Add the background color, if configured.
    $bg_color = $this->configuration['bgcolor_active'] ? $this->configuration['bgcolor_value'] : NULL;
    if ($bg_color) {
      list($r, $g, $b) = sscanf($bg_color, "#%02x%02x%02x");
      $bg_image = imagecreatetruecolor($width, $height);
      imagefill($bg_image, 0, 0, imagecolorallocate($bg_image, $r, $g, $b));
      imagecopy($bg_image, $raster_image, 0, 0, 0, 0, $width, $height);
    }

    // Create a new unmanaged png thumbnail file.
    if (!imagepng($bg_color ? $bg_image : $raster_image, $sourceUri . '.png', 9)) {
      $this->logger->warning($this->t('Could not create png from svg.'));
      return NULL;
    }

    // Create a managed file object.
    $file = File::create([
      'uri' => $sourceUri . '.png',
      'status' => 1,
    ]);
    try {
      $file->save();
      return $file;
    } catch (EntityStorageException $e) {
      $this->logger->warning($this->t('Could not create thumbnail file entity.'));
      return NULL;
    }

  }

}
